--[[
Name: MythicPlayerInfo

Copyright 2018 Amegez-Dalaran,MerguezChipo-Dalaran

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
]]

ACTIVITY_IDS={}
ACTIVITY_IDS[197]=459;
ACTIVITY_IDS[198]=460;
ACTIVITY_IDS[199]=463;
ACTIVITY_IDS[200]=461;
ACTIVITY_IDS[206]=462;
ACTIVITY_IDS[207]=464;
ACTIVITY_IDS[208]=465;
ACTIVITY_IDS[209]=467;
ACTIVITY_IDS[210]=466;
ACTIVITY_IDS[227]=471;
ACTIVITY_IDS[233]=476;
ACTIVITY_IDS[234]=473;
ACTIVITY_IDS[239]=486;
ACTIVITY_IDS[244]=502;
ACTIVITY_IDS[245]=518;
ACTIVITY_IDS[246]=526;
ACTIVITY_IDS[247]=510;
ACTIVITY_IDS[248]=530;
ACTIVITY_IDS[249]=514;
ACTIVITY_IDS[250]=504;
ACTIVITY_IDS[251]=507;
ACTIVITY_IDS[252]=522;
ACTIVITY_IDS[353]=534;

MythicPlayerInfo = {};

C_ChatInfo.RegisterAddonMessagePrefix("MythicPlayerInfo");
local defaultGTTFontName = "";
local defaultGTTFontSize = 0;

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(e,EventName,Data1,Data2)
	if(EventName == "CHAT_MSG_ADDON_LOGGED" and Data1 == "MythicPlayerInfo")then
		if string.find(Data2, "01", 1) == 1 then -- Collect player mythic info
			MPI_setPlayerInfo(string.sub(Data2,4));
		elseif string.find(Data2, "02", 1) == 1 then -- Request player mythic info
			MPI_sendPlayerInfo(string.sub(Data2,4));
			if IsInGroup("player") and UnitIsGroupLeader("player") then
				MPI_sharePlayerInfo(Data2);
			end
		end
	elseif (EventName == "LFG_LIST_APPLICATION_STATUS_UPDATED" and Data2 == "applied")then
		local _, _, _, _, _, _, _, _, _, _, _, _, leader = C_LFGList.GetSearchResultInfo(Data1);
		C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", MPI_GetPlayerInfo(), "WHISPER", leader);
		if IsInGroup("player") and UnitIsGroupLeader("player") then
			MPI_sharePlayerInfo("02#"..leader);
		end
	elseif (EventName == "PLAYER_ENTERING_WORLD")then
		C_MythicPlus.RequestCurrentAffixes();
		C_MythicPlus.RequestMapInfo();
		C_MythicPlus.RequestRewards();
		MPI_init();
	end
end)
f:RegisterEvent("CHAT_MSG_ADDON_LOGGED");
f:RegisterEvent("LFG_LIST_APPLICATION_STATUS_UPDATED");
f:RegisterEvent("PLAYER_ENTERING_WORLD");

function MPI_ShowToolTip(self)
	local name = C_LFGList.GetApplicantMemberInfo(self:GetParent().applicantID,1);
	local _, activityID = C_LFGList.GetActiveEntryInfo();
	if not MythicPlayerInfo[name] then
		MythicPlayerInfo[name] = {};
	end
	MPI_SetTooltipPlayerInfo(name,activityID);
end

function MPI_HideToolTip(self)
	MPI_GameTooltip_Hide();
	MPI_GameTooltip_ClearLines();
end

function MPI_setPlayerInfo(data)
	local info = {strsplit("#",data)};
	MythicPlayerInfo[info[1]] = {};
	MythicPlayerInfo[info[1]].classId = info[2];
	MythicPlayerInfo[info[1]].level = info[3];
	MythicPlayerInfo[info[1]].specId = info[4];
	MythicPlayerInfo[info[1]].iLvl = info[5];
	MythicPlayerInfo[info[1]].data = {};
	MythicPlayerInfo[info[1]].group = {};
	for i=6,#info,4 do
		if not string.find(info[i], "P_", 1) then
			table.insert(MythicPlayerInfo[info[1]].data,{info[i],info[i+1],info[i+2],info[i+3]});
		else
			for j=i,i+4 do
				if info[j] then
					table.insert(MythicPlayerInfo[info[1]].group,string.sub(info[j], 3));
				end
			end
		end
	end
end

function MPI_GetPlayerInfo()
	local playerName = UnitFullName("player");
	local ilvl, ilvle, _ = GetAverageItemLevel();
	local _, _, classIndex = UnitClass("player");
	local specID = GetSpecializationInfo(GetSpecialization());
	local level = UnitLevel("player");
	local playerInfo = "01#"..playerName.."#"..classIndex.."#"..level.."#"..specID.."#"..ilvle.."/"..ilvl ;
	for k,v in pairs(ACTIVITY_IDS) do
		local seasonBestDurationSec, seasonBestLevel, _, _, members = C_MythicPlus.GetSeasonBestForMap(k);
		if seasonBestLevel then
			local role = 0;
			for i, member in ipairs(members) do
				if (member.name and member.name == playerName) then
					role = member.specID;
				end
			end
			playerInfo = string.format("%s#%s#%s#%s#%s",playerInfo,k,seasonBestDurationSec,seasonBestLevel,role);
		end
	end
	if IsInGroup("player") and UnitIsGroupLeader("player") then
		for i=1,4 do
			if UnitName("party"..i) then
				local partyName = UnitFullName("party"..i);
				playerInfo = playerInfo.."#".."P_"..partyName;
			end
		end
	end
	TEST = playerInfo;
	return playerInfo;
end

function MPI_sendPlayerInfo(requestFrom)
	C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", MPI_GetPlayerInfo(), "WHISPER", requestFrom);
end

function MPI_requestPlayerInfo(requestToPlayer)
	local playerName = string.format("%s-%s",UnitFullName("player"));
	C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", "02#"..playerName, "WHISPER", requestToPlayer);
end

function MPI_sharePlayerInfo(requestToPlayer)
	for i=1,4 do
        if (UnitName("party"..i)) then
			local partyName,realmName = UnitFullName("party"..i);
			if realmName and realmName ~= "" then
				partyName = string.format("%s-%s",UnitFullName("party"..i));
			end
			C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", requestToPlayer, "WHISPER", partyName);
		end
	end
end

function MPI_SetTooltipPlayerInfo(name,activityID)
	if MythicPlayerInfo[name].group and #MythicPlayerInfo[name].group > 0 then
		MPI_SetTooltipPlayerInfoExtend(name,nil,activityID);
	else
		MPI_SetTooltipPlayerInfoExtend(name);
	end
end

function MPI_SetTooltipPlayerInfoExtend(name,noclear,activityID)
	if not noclear then
		MPI_GameTooltip_ClearLines();
		MPI_GameTooltip.name = name;
		if activityID then
			MPI_GameTooltip.activityID = activityID;
		end
	end
	
	if not MythicPlayerInfo[name] or not MythicPlayerInfo[name].data then
		MPI_GameTooltip_AddLine("MythicPlayerInfo",RAID_CLASS_COLORS.ROGUE.colorStr);
		MPI_GameTooltip_AddLine(name.." doesn't have the addon");
		MPI_GameTooltip_Show();
		return;
	end
	local localizedClass, class = GetClassInfo(MythicPlayerInfo[name].classId);
	local level = MythicPlayerInfo[name].level;
	local itemLevel = MythicPlayerInfo[name].iLvl;
	local classTextColor = grayedOut and GRAY_FONT_COLOR or RAID_CLASS_COLORS[class];
	local _, nameSpec, _, _ , role = GetSpecializationInfoByID(MythicPlayerInfo[name].specId);
	local texture = MPI_getRoleTexture(role);

	if not noclear then
		MPI_GameTooltip_AddLine("MythicPlayerInfo",RAID_CLASS_COLORS.ROGUE.colorStr);
	end
	MPI_GameTooltip_AddLine(string.format("%s (%s)",MYTHIC_PLUS_LEADER_BOARD_NAME_ICON:format(texture,name),nameSpec),RAID_CLASS_COLORS[class].colorStr);
	MPI_GameTooltip_AddLine(string.format(UNIT_TYPE_LEVEL_TEMPLATE, level, localizedClass),RAID_CLASS_COLORS[class].colorStr);
	if not string.find(itemLevel, "/", 1) then -- V0.7 => V0.8
		MPI_GameTooltip_AddLine(string.format(LFG_LIST_ITEM_LEVEL_CURRENT, itemLevel));
	else
		MPI_GameTooltip_AddLine(string.format(LFG_LIST_ITEM_LEVEL_CURRENT.."/%s", strsplit("/",itemLevel)));
	end
		
	local score = 0;
	local str = {};
	if MythicPlayerInfo[name] then
		for k,v in pairs(MythicPlayerInfo[name].data) do
			local mname,_,timer = C_ChallengeMode.GetMapUIInfo(v[1]);
			score = score + ((v[3]*10)+((timer-v[2])/(timer/10)));

			if not (activityID and ACTIVITY_IDS[tonumber(v[1])] ~= activityID) then
				mname = MPI_formatString(mname,30);
				local oyu = 0;
				if timer-v[2] >= 0 then
					oyu = string.format("|cffabd473Under by %s|r",GetTimeStringFromSeconds(timer-v[2]));
				else
					oyu = string.format("|cffc41f3bOver  by %s|r",GetTimeStringFromSeconds(v[2]-timer));
				end
				local dtime = GetTimeStringFromSeconds(v[2]);
				local _, nameSpec = GetSpecializationInfoByID(tonumber(v[4]));
				nameSpec = MPI_formatString(nameSpec,15);
				local role = GetSpecializationRoleByID(tonumber(v[4]));
				local texture = MPI_getRoleTexture(role);
				if (role == "TANK") then
					texture = CreateAtlasMarkup("roleicon-tiny-tank");
				elseif (role == "DAMAGER") then
					texture = CreateAtlasMarkup("roleicon-tiny-dps");
				elseif (role == "HEALER") then
					texture = CreateAtlasMarkup("roleicon-tiny-healer");
				end
				local lvel = MPI_formatString(v[3],2);
				str[k] = string.format("%s %s +%s %s (%s)",mname,MYTHIC_PLUS_LEADER_BOARD_NAME_ICON:format(texture,nameSpec),lvel,dtime,oyu);
			end
		end
		MPI_GameTooltip_AddLine(string.format("Score : %.2f",score),RAID_CLASS_COLORS.MONK.colorStr);
		for k,v in pairs(str) do
			MPI_GameTooltip_AddLine(v);		
		end
		if #MythicPlayerInfo[name].data == 0 then
			MPI_GameTooltip_AddLine("No Mythic+ run");
		end
	end
	for i=1,#MythicPlayerInfo[name].group do
		MPI_SetTooltipPlayerInfoExtend(MythicPlayerInfo[name].group[i],true,activityID);
	end
	
	if not MPI_GameTooltip.ShowFull and not noclear and #MythicPlayerInfo[name].group > 0 then
		MPI_GameTooltip_AddLine("");
		MPI_GameTooltip_AddLine("Press Left Ctrl for more details",RAID_CLASS_COLORS.MONK.colorStr);
	end
	
	MPI_GameTooltip_Show();
end

function MPI_ShowToolTipKeyUp(self, key)
	if key=="LCTRL" and MPI_GameTooltip.ShowFull then
		MPI_GameTooltip_Hide();
		MPI_GameTooltip.ShowFull = nil;
		self:SetPropagateKeyboardInput(true);
		MPI_SetTooltipPlayerInfo(MPI_GameTooltip.name,MPI_GameTooltip.activityID);
	end
end

function MPI_ShowToolTipKeyDown(self, key)
	if key=="LCTRL" and not MPI_GameTooltip.ShowFull then
		MPI_GameTooltip_Hide();
		MPI_GameTooltip.ShowFull = true;
		self:SetPropagateKeyboardInput(key~="LCTRL");
		MPI_SetTooltipPlayerInfo(MPI_GameTooltip.name);
	end
end

function MPI_getRoleTexture(role)
	local texture = CreateAtlasMarkup("roleicon-tiny-dps");
	if (role == "TANK") then
		texture = CreateAtlasMarkup("roleicon-tiny-tank");
	elseif (role == "DAMAGER") then
		texture = CreateAtlasMarkup("roleicon-tiny-dps");
	elseif (role == "HEALER") then
		texture = CreateAtlasMarkup("roleicon-tiny-healer");
	end
	return texture;
end

function MPI_formatString(str, maxChar)
	local specialChars = strtrim(str,'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890 -_\'!');
	local toRemove = floor(#str-(#specialChars/2));
	return str..string.rep(' ', maxChar-toRemove);
end

function MPI_ButtonShow(self)
	for i=1,#self.Members do
		local b = self.Members[i];
		b:HookScript("OnEnter", MPI_ShowToolTip);
		b:HookScript("OnLeave", MPI_GameTooltip_Hide);
		b:SetScript("OnKeyDown",MPI_ShowToolTipKeyDown);
		b:SetScript("OnKeyUp",MPI_ShowToolTipKeyUp);
	end
end

function MPI_GameTooltip_AddLine(text,color)
	if color then
		text = "|c"..color..text.."|r";
	else
		text = "|c"..RAID_CLASS_COLORS.PRIEST.colorStr..text.."|r";
	end
	if(MPI_GameTooltip.text:GetText())then
		MPI_GameTooltip.text:SetText(MPI_GameTooltip.text:GetText().."\r\n"..text);
	else
		MPI_GameTooltip.text:SetText(text);
	end
end

function MPI_GameTooltip_ClearLines()
	MPI_GameTooltip.text:SetText("");
end

function MPI_GameTooltip_Hide()
	MPI_GameTooltip:Hide();
end

function MPI_GameTooltip_Show()
	MPI_GameTooltip:ClearAllPoints();
	MPI_GameTooltip:SetWidth(MPI_GameTooltip.text:GetStringWidth()+10);
	MPI_GameTooltip:SetHeight(MPI_GameTooltip.text:GetStringHeight()+20);
	if MPI_GameTooltip.ShowFull then
		MPI_GameTooltip:SetPoint("TOPLEFT",UIParent,"TOPLEFT",0,0);
	else
		MPI_GameTooltip:SetPoint("TOPLEFT",GroupFinderFrame,"BOTTOMLEFT",0,-30);
	end
	MPI_GameTooltip:Show();
end

hooksecurefunc("LFGListUtil_SetSearchEntryTooltip", function(tooltip, resultID, autoAcceptOption)
	MPI_GameTooltip:Hide();
	local id, activityID, _, _, _, _, _, _, _, _, _, _, leaderName ,numMembers = C_LFGList.GetSearchResultInfo(resultID);
	if leaderName then
		local leaderShort = {strsplit("-",leaderName)};
		MPI_GameTooltip_ClearLines();
		if not MythicPlayerInfo[leaderShort[1]] then
			MythicPlayerInfo[leaderShort[1]] = {};
			MPI_GameTooltip_AddLine("MythicPlayerInfo",RAID_CLASS_COLORS.ROGUE.colorStr);
			MPI_GameTooltip_AddLine("Looking for "..leaderName.." infos ...");
			MPI_GameTooltip_Show();
			MPI_requestPlayerInfo(leaderName);
			C_Timer.After(1,function()
				MPI_SetTooltipPlayerInfo(leaderShort[1],activityID);
			end)
		else
			MPI_SetTooltipPlayerInfo(leaderShort[1],activityID);
		end
	end
end)

function MPI_init()
	for i=1,14 do
		local b = _G["LFGListApplicationViewerScrollFrameButton"..i];
		b:HookScript("OnShow", MPI_ButtonShow);
	end

	for i=1,#_G["LFGListSearchPanelScrollFrame"].buttons do
		local b = _G["LFGListSearchPanelScrollFrameButton"..i];
		b:HookScript("OnLeave", MPI_GameTooltip_Hide);
		b:SetScript("OnKeyDown",MPI_ShowToolTipKeyDown);
		b:SetScript("OnKeyUp",MPI_ShowToolTipKeyUp);
		b:SetPropagateKeyboardInput(true);
	end

	local lfgframe = LFGListFrame.ApplicationViewer.UnempoweredCover;
	lfgframe:EnableMouse(false);
	lfgframe:EnableMouseWheel(false);
	lfgframe:SetToplevel(false);
	
	if not MPI_GameTooltip then
		MPI_GameTooltip = CreateFrame("Frame",nil,GroupFinderFrame);
		MPI_GameTooltip:SetFrameStrata("TOOLTIP");
		MPI_GameTooltip:SetPoint("BOTTOMLEFT",0,0);
		MPI_GameTooltip:SetSize(700,400);
		MPI_GameTooltip:SetBackdrop({
			 bgFile = "Interface/ChatFrame/ChatFrameBackground",
			 edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			 tile = true, tileSize = 32, edgeSize = 16,
			 insets = { left = 1, right = 1, top = 1, bottom = 1 }});
		MPI_GameTooltip:SetBackdropColor(0,0,0,1);
		MPI_GameTooltip.text = MPI_GameTooltip:CreateFontString("MPI_GameTooltipText","ARTWORK",MPI_GameTooltip);
	end
	MPI_GameTooltip.text:SetPoint("TOPLEFT", 10, -10);
	MPI_GameTooltip.text:SetTextColor(0.85, 0.85, 0.85, 1);
	MPI_GameTooltip.text:SetFont("Interface\\AddOns\\MythicPlayerInfo\\fonts\\MonospaceBold.ttf", 12);
	MPI_GameTooltip.text:SetJustifyH("LEFT");
	MPI_GameTooltip.text:SetText("");
	MPI_GameTooltip:Hide();
end

